%{
#include <stdio.h>
%}

%union {
	double	real; /* real value */
	int	integer; /* integer value */
}

%token	<real>		REAL
%token	<integer>	INTEGER

%type	<real>		rexpr
%type	<integer>	iexpr

%left '+' '-'
%left '%'
%left '*' '/'
%left	UMINUS

%%

lines	:
	| lines line
	;

line	: '\n'
	| iexpr '\n'
	  { printf("%d\n",$1); }
	| rexpr '\n'
	  { printf("%15.8lf\n",$1); }
	;


iexpr	: INTEGER
	| iexpr '+' iexpr
	  { $$ = $1 + $3 ; }
	| iexpr '-' iexpr
	  { $$ = $1 - $3 ; }
	| iexpr '*' iexpr
	  { $$ = $1 * $3 ; }
	| iexpr '%' iexpr
	  { if($3)
		$$ = $1 % $3 ;
	    else {
		yyerror("divide by zero");
	    }
	  }
	| iexpr '/' iexpr
	  { if($3)
		$$ = $1 / $3 ;
	    else {
		yyerror("divide by zero");
	    }
	  }
	| '-' iexpr %prec UMINUS
	  { $$ = - $2 ; }
	| '(' iexpr ')'
	  { $$ = $2 ; }
	;

rexpr	: REAL
	| rexpr '+' rexpr
	  { $$ = $1 + $3 ; }
	| rexpr '-' rexpr
	  { $$ = $1 - $3 ; }
	| rexpr '*' rexpr
	  { $$ = $1 * $3 ; }
	| rexpr '/' rexpr
	  { if($3)
		$$ = $1 / $3 ;
	    else {
		yyerror("divide by zero");
	    }
	  }
	| '-' rexpr %prec UMINUS
	  { $$ = - $2 ; }
	| '(' rexpr ')'
	  { $$ = $2 ; }
	| iexpr '+' rexpr
	  { $$ = (double)$1 + $3 ; }
	| iexpr '-' rexpr
	  { $$ = (double)$1 - $3 ; }
	| iexpr '*' rexpr
	  { $$ = (double)$1 * $3 ; }
	| iexpr '/' rexpr
	  { if($3)
		$$ = (double)$1 / $3 ;
	    else {
		yyerror("divide by zero");
	    }
	  }
	| rexpr '+' iexpr
	  { $$ = $1 + (double)$3 ; }
	| rexpr '-' iexpr
	  { $$ = $1 - (double)$3 ; }
	| rexpr '*' iexpr
	  { $$ = $1 * (double)$3 ; }
	| rexpr '/' iexpr
	  { if($3 != 0)
		$$ = $1 / (double)$3 ;
	    else {
		yyerror("divide by zero");
	    }
	  }
	;

%%

